import "Turbine.Gameplay";

KeyManager = class();

function KeyManager:Constructor()
    -- listeners
    self.listeners = {};
    
    self.locked = true;
    self.hidden = false;
    
    -- hack control to get update/key events, remove when we can get updates without being a control
    self.update = Turbine.UI.Control();
    self.update:SetVisible(false);
    self.update:SetWantsKeyEvents(true);
    self.update.KeyDown = function(sender, args)
        if (args.Action == 0x1000007B) then
            -- unlock the control for movement
            self:FireUnlock();
        elseif (args.Action == 0x100000B3) then
            -- hide ui elements
            self:FireHidden();
        end
    end
end

function KeyManager:RegisterUnloadEvent(pluginName, callback)
    self.update:SetWantsUpdates(true);
    self.update.Update = function(sender, args)
        if (Plugins[pluginName] ~= nil) then
            Plugins[pluginName].Unload = callback;
            self.update:SetWantsUpdates(false);
        end
    end
end

function KeyManager:IsLocked()
    return self.locked;
end

function KeyManager:IsHidden()
    return self.hidden;
end

function KeyManager:AddListener(listener)
    -- make sure it isn't in the list already
    self:RemoveListener(listener);
    table.insert(self.listeners, listener);
end

function KeyManager:RemoveListener(listener)
    local i;
    for i = 1, table.getn(self.listeners) do
        if (self.listeners[i] == listener) then
            table.remove(self.listeners, i);
            break;
        end
    end
end

function KeyManager:FireUnlock()
    self.locked = not self.locked;
    local i;
    for i = 1, table.getn(self.listeners) do
        if (type(self.listeners[i].UiLocked) == "function") then
            self.listeners[i]:UiLocked(self.locked);
        end
    end
end

function KeyManager:FireHidden()
    self.hidden = not self.hidden;
    local i;
    for i = 1, table.getn(self.listeners) do
        if (type(self.listeners[i].UiHidden) == "function") then
            self.listeners[i]:UiHidden(self.hidden);
        end
    end
end

-- define a global instance
KeyManager.instance = KeyManager();
