import "TurbinePlugins.Vitals";

VitalsWindow = class( Turbine.UI.Window );
VitalsWindow.moraleBar = VitalsBar();
VitalsWindow.powerBar = VitalsBar();

function VitalsWindow:Constructor()
	Turbine.UI.Window.Constructor( self );
	
	self.flashing = false;
	self.flashTime = 0;
	self.flashLimit = 0.33;
	self.locked = Faflaf.Mover.KeyManager.instance:IsLocked();
    self.hidden = Faflaf.Mover.KeyManager.instance:IsHidden();
	
	self:SetBackColor( Turbine.UI.Color( 0.6, 0, 0, 0 ) );
	
	-- Construct the morale bar.
	self.moraleBar:SetParent( self );
	self.moraleBar:SetZOrder( 100 );
	self.moraleBar:SetColor( Turbine.UI.Color( 1, 0.1, 1, 0.1 ) );
	self.moraleBar:SetMouseVisible( false );
	
	-- Construct the power bar.
	self.powerBar:SetParent( self );
	self.powerBar:SetZOrder( 100 );
	self.powerBar:SetColor( Turbine.UI.Color( 1, 0.1, 0.5, 1 ) );
	self.powerBar:SetMouseVisible( false );

	self.levelLabel = Turbine.UI.Label();
	self.levelLabel:SetParent( self.moraleBar );
	self.levelLabel:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleLeft );
	self.levelLabel:SetForeColor( Turbine.UI.Color(245/255, 222/255, 147/255) );
	self.levelLabel:SetFont( Turbine.UI.Lotro.Font.TrajanPro14 );
	self.levelLabel:SetFontStyle( Turbine.UI.FontStyle.Outline );
	self.levelLabel:SetOutlineColor( Turbine.UI.Color( 0, 0, 0 ) );
	self.levelLabel:SetMouseVisible( false );

	self.nameLabel = Turbine.UI.Label();
	self.nameLabel:SetParent( self.moraleBar );
	self.nameLabel:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleLeft );
	self.nameLabel:SetForeColor( Turbine.UI.Color(245/255, 222/255, 147/255) );
	self.nameLabel:SetFont( Turbine.UI.Lotro.Font.TrajanPro14 );
	self.nameLabel:SetFontStyle( Turbine.UI.FontStyle.Outline );
	self.nameLabel:SetOutlineColor( Turbine.UI.Color( 0, 0, 0 ) );
	self.nameLabel:SetMouseVisible( false );

	-- Hook into the player to receive updates about player morale and power.
	self.player = Turbine.Gameplay.LocalPlayer()
	self.levelLabel:SetText( self.player:GetLevel() );
	self.nameLabel:SetText( self.player:GetName() );
	
	self.player.NameChanged = function( sender, args )
		self.nameLabel:SetText( self.player:GetName() );
	end
	
	self.player.LevelChanged = function( sender, args )
		self.levelLabel:SetText( self.player:GetLevel() );
	end
	
	self.player.MoraleChanged = function( sender, args )
		self:UpdateMorale()
	end
	
	self.player.MaxMoraleChanged = function( sender, args )
		self:UpdateMorale()
	end
	
	self.player.BaseMaxMoraleChanged = function( sender, args )
		self:UpdateMorale()
	end
	
	self.player.PowerChanged = function( sender, args )
		self:UpdatePower()
	end
	
	self.player.MaxPowerChanged = function( sender, args )
		self:UpdatePower()
	end
	
	self.player.BaseMaxPowerChanged = function( sender, args )
		self:UpdatePower()
	end
	
	Faflaf.Mover.KeyManager.instance:AddListener( self );
	-- movement listeners
    self.DragEnd = function(sender)
        local left, top = self:GetPosition();
        settings.WindowLeft, settings.WindowTop = self:GetPosition();
		settings:Save();
    end

	self:UpdateVitals();
	self:PerformLayout();

	if( not self.locked ) then
        self:UiLocked( self.locked );
    end
end

function VitalsWindow:MouseDown( args )
	if( args.Button == Turbine.UI.MouseButton.Right ) then
		vitalsMenu:ShowMenu();
	end
end

function VitalsWindow:UiLocked( locked )
    self.locked = locked;
    if( not self.locked ) then
        self.dragBar = Faflaf.Mover.DragBar( self, "VitalBars" );
        self.dragBar:SetBarOnTop( true );
        self.dragBar:SetBarVisible( not self.hidden );
        self.dragBar:SetDraggable( true );
        
        self:UpdateVisibility();
    else
        if( self.dragBar ~= nil ) then
            self.dragBar:SetDraggable( false );
            self.dragBar:SetTarget( nil );
            self.dragBar = nil;
        end
        
        self:SetBackColor(Turbine.UI.Color(0, 0, 0, 0));
        self:PerformLayout();
    end
end

function VitalsWindow:UiHidden(hidden)
    self.hidden = hidden;
    if( self.dragBar ~= nil ) then
        self.dragBar:SetVisible( not hidden );
    end
    self:UpdateVisibility();
end

function VitalsWindow:UpdateVisibility()
    local visible;
    if( self.hidden ) then
        visible = false;
    elseif( self.locked ) then
        visible = true;
    else
        visible = false;
    end
    self:SetVisible( visible );
end

function VitalsWindow:IsStacked()
	return self.stacked;
end

function VitalsWindow:SetStacked( value )
	self.stacked = value;
end

function VitalsWindow:UpdateMorale()
	local morale        = self.player:GetMorale();
	local maxMorale     = self.player:GetMaxMorale();
	local baseMaxMorale = self.player:GetBaseMaxMorale();

	self.moraleBar:SetValue( morale );
	self.moraleBar:SetMax( maxMorale );
	self.moraleBar:SetBaseMax( baseMaxMorale );

	local moraleRatio = ( morale / maxMorale );

	local newR = 1 - moraleRatio
	local newG = moraleRatio
	local scaleFactor = 1

	if ( newR < newG ) then
		scaleFactor = 1 / newG
	else
		scaleFactor = 1 / newR
	end

	newR = newR * scaleFactor;
	newG = newG * scaleFactor;

	self.moraleBar:SetColor( Turbine.UI.Color( 1.0, newR, newG, 0.0 ) );

	if ( ( not self.flashing ) and ( moraleRatio < self.flashLimit ) ) then
		self.flashing = true;
		self.flashTime = Turbine.Engine.GetGameTime();
		self:SetWantsUpdates( true );
	end
end

function VitalsWindow:UpdatePower()
	local power = self.player:GetPower()
	self.powerBar:SetValue( power );

	local maxPower = self.player:GetMaxPower()
	self.powerBar:SetMax( maxPower );

	local baseMaxPower = self.player:GetBaseMaxPower()
	self.powerBar:SetBaseMax( baseMaxPower );
end

function VitalsWindow:UpdateVitals()
	self:UpdateMorale();
	self:UpdatePower();
end

function VitalsWindow:Update( args )
	local morale    = self.player:GetMorale();
	local maxMorale = self.player:GetMaxMorale();

	local moraleRatio = ( morale / maxMorale );
	local flashAmount = 0;

	if ( moraleRatio >= self.flashLimit ) then
		self.flashing = false;
		self:SetWantsUpdates( false );
	else
		flashAmount = ( math.sin( 12 * ( Turbine.Engine.GetGameTime() - self.flashTime ) ) + 1 ) / 2;
	end

	local newR = 1 - moraleRatio;
	local newG = moraleRatio;
	local scaleFactor = 1;

	if ( newR < newG ) then
		scaleFactor = 1 / newG
	else
		scaleFactor = 1 / newR
	end

	newR = newR * scaleFactor;
	newG = newG * scaleFactor;

	newR = newR + flashAmount * ( 1 - newR );
	newG = newG + flashAmount * ( 1 - newG );

	self.moraleBar:SetColor( Turbine.UI.Color( 1.0, newR, newG, 0.0 ) );
end

function VitalsWindow:PerformLayout()
	self:SetStacked( settings.IsStacked );
	self:SetPosition( settings.WindowLeft, settings.WindowTop );
	
	self.moraleBar:SetPosition( 1, 1 );
	self.moraleBar:SetSize( settings.MoraleWidth, 12 );
	self.moraleBar:SetShowPercentage( settings.UseMoralePercentage );
	self.moraleBar:SetTextVisible( settings.IsMoraleTextVisible );
	
	self.powerBar:SetSize( settings.PowerWidth, 12 );
	self.powerBar:SetShowPercentage( settings.UsePowerPercentage );
	self.powerBar:SetTextVisible( settings.IsPowerTextVisible );
	if( self:IsStacked() ) then
		local width;
		if( self.moraleBar:GetWidth() > self.powerBar:GetWidth() ) then
			width = self.moraleBar:GetWidth()+2;
		else
			width = self.powerBar:GetWidth()+2;
		end
		self:SetSize( width, 27 );	
		self.powerBar:SetPosition( 1, 14 );
	else
		self:SetSize( (self.moraleBar:GetWidth() + self.powerBar:GetWidth() + 3), 14 );
		self.powerBar:SetPosition( (self.moraleBar:GetWidth()+2), 1 );
	end
	
	self.levelLabel:SetVisible( settings.IsLevelVisible );
	self.levelLabel:SetPosition( 2, -1 );
	self.levelLabel:SetSize( 20, 14 );
	self.levelLabel:SetZOrder( 201 );
	if( self.levelLabel:IsVisible() ) then
		self.nameLabel:SetPosition( (self.levelLabel:GetLeft()+25), -1 );
	else
		self.nameLabel:SetPosition( 2, -1 );
	end
	self.nameLabel:SetVisible( settings.IsNameVisible );
	self.nameLabel:SetSize( 300, 14 );
	self.nameLabel:SetZOrder( 201 );

	self.moraleBar:Update();
	self.powerBar:Update();

	self:UpdateVisibility();
end